@@ -16,22 +16,22 @@
 #' @title Plot of channel intensity marginal densities
 #' @description `ggplotMarginalDensities` uses ggplot2 
 #' to draw plots of marginal densities of selected channels of a flowSet.
-#' If the flowSet contains several flowFrames, all events are concatenated 
-#' together.    
-#' By default, a pseudo Rsquare projection quality indicator, 
-#' and the number of dimensions of the MDS projection are provided in sub-title
+#' If the flowSet contains several flowFrames, events are concatenated 
+#' together per group, or all together in the absence of groups.     
 #' @param x a `flowCore::flowSet` (or a single `flowCore::flowFrame`)
-#' @param sampleSubset (optional) a logical vector, of size `nrow(pData)`, 
-#' which is by construction the nb of samples, indicating which samples to keep 
-#' in the plot. Typically it is obtained through the evaluation of 
-#' a logical condition on `pData` rows.   
-#' @param channels (optional) 
-#' @param pDataForColour (optional) which `phenoData(fs)` variable
+#' @param sampleSubset (optional) a logical vector, of the same length as `x`, 
+#' indicating which flow frames to keep in the plot. 
+#' Typically it is obtained through the evaluation of a logical condition 
+#' the rows of `phenoData(fs)`.   
+#' @param channels (optional) - can be indices, or channel names, or markers.
+#' @param pDataForColour (optional) which variable of `phenoData(fs)` 
 #' will be used as colour aesthetic. Should be a character.
-#' @param pDataForGroup (optional) which `phenoData(fs)` variable
+#' @param pDataForGroup (optional) which variable of `phenoData(fs)` 
 #' will be used as group aesthetic. Should be a character.
+#' A separate marginal density will be calculated for each group and 
+#' overlaid on the same channel density plots.
 #' @param nEventInSubsample how many event to take 
-#' (per flowFrame of the flowSet).
+#' (per flowFrame of the flowSet) for marginal density approximation.
 #' @param seed if not null, used in subsampling.
 #' @param transList a `flowCore::transformList` that will be applied 
 #' before plotting.
@@ -168,28 +168,34 @@ ggplotMarginalDensities <- function(
         }
     }
     
+    nChannels <- length(channels)
     
-    channelLabels <- vapply(channels,
+    channelInfos <- lapply(channels,
                        FUN = function(ch, fr){
                            chmk <- flowCore::getChannelMarker(fr, ch)
                            if (is.null(chmk)) {
                                stop("channel ", ch, " not found in expr matrix")
                            }
-                           label <- chmk$name
+                           name <- chmk$name
+                           label <- name
                            if (!is.na(chmk$desc) && 
                                !toupper(chmk$desc) == "EMPTY"){
                                label <- chmk$desc
                            }
-                           label
+                           list(name = name, label = label)
                        },
-                       FUN.VALUE = " ",
                        fr = fs[[1]])
     
-    nChannels <- length(channels)
+    channelNames <- vapply(channelInfos,
+                           FUN = function(chi) chi$name,
+                           FUN.VALUE = " ")
+    channelLabels <- vapply(channelInfos,
+                            FUN = function(chi) chi$label,
+                            FUN.VALUE = " ")
     
     DFList <- mapply(
         seq_len(nSamples),
-        FUN = function(i, fs, channelLabels, nEventInSubsample, seed){
+        FUN = function(i, fs, channels, channelLabels, nEventInSubsample, seed){
             nEvents <- flowCore::nrow(fs[[i]])
             chosenEvents <- 0
             if (nEventInSubsample < nEvents) {
@@ -215,6 +221,7 @@ ggplotMarginalDensities <- function(
         },
         MoreArgs = list(
             fs = fs,
+            channels = channelNames,
             channelLabels = channelLabels,
             nEventInSubsample = nEventInSubsample,
             seed = seed),
@@ -258,6 +265,84 @@ ggplotMarginalDensities <- function(
     p 
 } 
 
+#' @title Plot of feature relative importance in distance
+#' @description `ggplotDistFeatureImportance` uses ggplot2 
+#' to provide a stacked bar plot of feature importance in a distance matrix.   
+#' @param distObj a DistSum object.
+#' @export
+#' 
+#' @seealso [pairwiseEMDDist]
+#' 
+#' @return a ggplot object
+#' 
+#' @examples
+#' 
+#' library(CytoPipeline)
+#' 
+#' data(OMIP021Samples)
+#' 
+#' # estimate scale transformations 
+#' # and transform the whole OMIP021Samples
+#' 
+#' transList <- estimateScaleTransforms(
+#'     ff = OMIP021Samples[[1]],
+#'     fluoMethod = "estimateLogicle",
+#'     scatterMethod = "linearQuantile",
+#'     scatterRefMarker = "BV785 - CD3")
+#' 
+#' OMIP021Trans <- CytoPipeline::applyScaleTransforms(
+#'     OMIP021Samples, 
+#'     transList)
+#' 
+#' # As there are only 2 samples in OMIP021Samples dataset,
+#' # we create artificial samples that are random combinations of both samples
+#' 
+#' ffList <- c(
+#'     flowCore::flowSet_to_list(OMIP021Trans),
+#'     lapply(3:5,
+#'            FUN = function(i) {
+#'                aggregateAndSample(
+#'                    OMIP021Trans,
+#'                    seed = 10*i,
+#'                    nTotalEvents = 5000)[,1:22]
+#'            }))
+#' 
+#' fsNames <- c("Donor1", "Donor2", paste0("Agg",1:3))
+#' names(ffList) <- fsNames
+#' 
+#' fsAll <- as(ffList,"flowSet")
+#' 
+#' flowCore::pData(fsAll)$type <- factor(c("real", "real", rep("synthetic", 3)))
+#' flowCore::pData(fsAll)$grpId <- factor(c("D1", "D2", rep("Agg", 3)))
+#' 
+#' # calculate all pairwise distances
+#' 
+#' pwDist <- pairwiseEMDDist(fsAll, 
+#'                              channels = c("FSC-A", "SSC-A"),
+#'                              verbose = FALSE)
+#'                              
+#' p <- ggplotDistFeatureImportance(pwDist)
+#'                              
+ggplotDistFeatureImportance <- function(distObj) {
+    DF <- distByFeature(distObj)
+    DF$featureName <- 
+        factor(DF$featureName, 
+               levels = DF$featureName[order(DF$percentage, decreasing = FALSE)])
+    DF$dum <- factor(1)
+    p <- ggplot(data = DF, aes(fill = .data[["featureName"]], 
+                               x = .data[["dum"]], 
+                               y = .data[["percentage"]])) + 
+        geom_bar(stat = "identity", colour = "black") + 
+        guides(fill = guide_legend(title = "feature")) + 
+        theme(axis.title.x=element_blank(),
+              axis.text.x=element_blank(),
+              axis.ticks.x=element_blank()) + 
+        labs(title = "Feature importance in distance matrix")
+    
+    p
+}
+
+
 #' @title Plot of Metric MDS object
 #' @description `ggplotSampleMDS` uses ggplot2 
 #' to provide plots of Metric MDS results.   
@@ -297,14 +382,16 @@ ggplotMarginalDensities <- function(
 #' If missing, point labels will be set equal to point names defined in 
 #' MDS object (if not NULL, otherwise no labels will be set). 
 #' @param pDataForAdditionalLabelling (optional) which `pData` variable(s)
-#' will be add to the ggplot mapping, as to make them available for 
-#' *plotly* tooltipping. Should be an array of character of maximum length 3.
+#' will be added to the ggplot mapping, as to make them available for 
+#' *plotly* tooltipping. Should be an array of character.
 #' Note this works only if biplot=FALSE, as biplots contain circle and arrows 
 #' that are currently not supported under `ggplotly`.
-#' @param sizeReflectingStress if TRUE, size of points will appear 
+#' @param pointSizeReflectingStress if TRUE, size of points will appear 
 #' proportional to stress by point, i.e. the bigger the sample point appears,
 #' the less accurate its representation is 
 #' (in terms of distances w.r.t. other points)
+#' @param pointSize size of all points on the plots - only when   
+#' `pointSizeReflectingStress` is FALSE. 
 #' @param title title to give to the plot
 #' @param displayPointLabels if TRUE, displays labels attached to points
 #' (see `pDataForLabels` for the setting of the label values)
@@ -329,7 +416,7 @@ ggplotMarginalDensities <- function(
 #' @param ... additional parameters passed to `ggrepel::geom_text_repel()` 
 #' (if used)
 #' @importFrom stats as.dist dist lm 
-#' @importFrom rlang .data
+#' @importFrom rlang .data sym
 #' 
 #' @export
 #' 
@@ -426,7 +513,7 @@ ggplotMarginalDensities <- function(
 #'     pDataForColour = "grpId",
 #'     pDataForLabel = "name",
 #'     pDataForShape = "type",
-#'     sizeReflectingStress = TRUE)
+#'     pointSizeReflectingStress = TRUE)
 #' 
 #' # try to associate axes with median of each channel
 #' # => use bi-plot
@@ -469,7 +556,8 @@ ggplotSampleMDS <- function(
         pDataForShape,
         pDataForLabel,
         pDataForAdditionalLabelling,
-        sizeReflectingStress = FALSE,
+        pointSize = 1.0,
+        pointSizeReflectingStress = FALSE,
         title = "Multi Dimensional Scaling",
         displayPointLabels = TRUE,
         pointLabelSize = 3.88,
@@ -621,38 +709,54 @@ ggplotSampleMDS <- function(
         nDim,
         ")")
     
-    mainAesMapping <- ggplot2::aes(
-        x = .data[["x"]],
-        y = .data[["y"]])
+    # Character vectors for aesthetic keys and values
+    mainAesKeys <- c("x", "y")
+    mainAesValues <- c("x", "y")
     
-    maxAdditionalLabellingMapping <- 3
     if (!missing(pDataForAdditionalLabelling) && !biplot) {
         if (!is.character(pDataForAdditionalLabelling)) {
             stop("pDataForAdditionalLabelling should be a character")
         }
-        if (length(pDataForAdditionalLabelling) > maxAdditionalLabellingMapping) {
-            stop("pDataForAdditionalLabelling length should be maximum 3")
-        }
-        if (length(pDataForAdditionalLabelling) >= 1) {
-            mainAesMapping <- 
-                c(mainAesMapping,
-                  ggplot2::aes(text = .data[[pDataForAdditionalLabelling[1]]]))
-        }
-        if (length(pDataForAdditionalLabelling) >= 2) {
-            mainAesMapping <- 
-                c(mainAesMapping,
-                  ggplot2::aes(text2 = .data[[pDataForAdditionalLabelling[2]]]))
-        }
-        if (length(pDataForAdditionalLabelling) >= 3) {
-            mainAesMapping <- 
-                c(mainAesMapping,
-                  ggplot2::aes(text3 = .data[[pDataForAdditionalLabelling[3]]]))
+        # remove from pDataForAdditionalLabelling :
+        # 'x', 'y', pDataForColour, pDataForShape, pDataForLabel and 'stress'
+        # to avoid possible duplicates
+        toRemove <- c("x", "y")
+        
+        if (!missing(pDataForColour)) {
+            toRemove <- union(toRemove, pDataForColour)
         }
         
-        # avoids error message: mapping should be created with `aes()`
-        attr(mainAesMapping, "class") <- "uneval"
+        if (!missing(pDataForShape)) {
+            toRemove <- union(toRemove, pDataForShape)
+        }
+        
+        if (displayPointLabels && !missing(pDataForLabel)) {
+            toRemove <- union(toRemove, pDataForLabel)
+        }
+        if (pointSizeReflectingStress) {
+            toRemove <- union(toRemove, "stress")
+        }
+        
+        pDataForAdditionalLabelling <- setdiff(pDataForAdditionalLabelling, 
+                                               toRemove)
+        
+        if (length(pDataForAdditionalLabelling) > 0) {
+            nAdditionalAes <- length(pDataForAdditionalLabelling) 
+            mainAesKeys <- c(mainAesKeys, 
+                             paste0("text", 1+seq_len(
+                                 length(pDataForAdditionalLabelling))))
+            mainAesValues <- c(mainAesValues, pDataForAdditionalLabelling)
+        }
     }
     
+    # dynamically create an aes mapping
+    mainAesList <- lapply(mainAesValues, FUN = function(val) {
+        rlang::sym(val)
+    })
+    names(mainAesList) <- mainAesKeys
+    
+    mainAesMapping <- do.call(aes, mainAesList)
+    
     rangeMin <- min(proj)
     rangeMax <- max(proj)
     axesLimits <- c(rangeMin, rangeMax)
@@ -689,7 +793,7 @@ ggplotSampleMDS <- function(
             geomPointMapping,
             ggplot2::aes(shape = .data[[shapeVar]]))
     }
-    if (sizeReflectingStress) {
+    if (pointSizeReflectingStress) {
         geomPointMapping <- c(
             geomPointMapping,
             ggplot2::aes(size = .data[["stress"]]))
@@ -700,7 +804,13 @@ ggplotSampleMDS <- function(
         attr(geomPointMapping, "class") <- "uneval"
     }
     
-    p <- p + ggplot2::geom_point(mapping = geomPointMapping)
+    if (pointSizeReflectingStress) {
+        p <- p + ggplot2::geom_point(mapping = geomPointMapping)    
+    } else {
+        p <- p + ggplot2::geom_point(
+            mapping = geomPointMapping,
+            size = pointSize)
+    }
     
     if (displayPointLabels) {
         if (!is.character(pDataForLabel)) {
