@@ -580,8 +580,13 @@ EMDDist <- function(
         colRange = colRange, 
         nRowBlock = nRowBlock)
     
+    # will store the feature names of the DistSum object at the end
+    usedFeatureNames <- NULL
+    
     if (unidimHistograms) {
-        
+        # currently , this portion of code is the only one that is used: 
+        # distance is calculated as the sum of uni-dimensional distances
+        # and we use histograms generation as a first step to compute those
         if (verbose){
             message("Pre-calculating all histograms...")
         }
@@ -681,6 +686,11 @@ EMDDist <- function(
         # reorganize multivariate distributions in a single list
         distrs <- unlist(distribBlockList, recursive = FALSE)
         
+        if (length(distrs) > 0) {
+            # should always be the case since nSamples > 0
+            usedFeatureNames <- colnames(distrs[[1]])   
+        }
+        
         if (verbose){
             message("Calculating pairwise distances between histograms...")
         }
@@ -697,44 +707,48 @@ EMDDist <- function(
             nRows <- length(rowSeq)
             nCols <- length(colSeq)
             
-            pwDistMat <- vapply(
-                seq_along(colSeq),
-                FUN = function(j) {
-                    pwDistCol <- vapply(
-                        seq_along(rowSeq),
-                        FUN = function(i) {
+            nChannel <- ncol(distrs[[1]])
+            channels <- colnames(distrs[[1]])
+            
+            # list of list of unidimensional vector of distances per channel
+            pwDistLL <- lapply(
+                seq_along(rowSeq),
+                FUN = function(i) {
+                    pwDistL <- lapply(
+                        seq_along(colSeq),
+                        FUN = function(j) {
+                            # we only calcuale upper triangular distance matrix
                             if (colSeq[j] > rowSeq[i]) {
-                                pwDist <- sum(.distFromUnidimHistograms(
+                                pwDist <- .distFromUnidimHistograms(
                                     breaks = breaks,
                                     distr1 =
                                         distrs[[which(
                                             rowColSeqUnion == rowSeq[i])]],
                                     distr2 =
                                         distrs[[which(
-                                            rowColSeqUnion == colSeq[j])]]))
+                                            rowColSeqUnion == colSeq[j])]])
                                 if (verbose) {
                                     message(
                                         "i = ", rowSeq[i],
                                         "; j = ", colSeq[j],
-                                        "; dist = ", round(pwDist, 12))
+                                        "; sum(dist) = ", 
+                                        round(sum(pwDist), 12))
                                 }
                             } else {
-                                pwDist <- 0.
+                                pwDist <- rep(0., nChannel)
+                                names(pwDist) <- colnames(distrs[[1]])
                             }
                             pwDist
-                        },
-                        FUN.VALUE = numeric(1))
-                    pwDistCol
-                },
-                FUN.VALUE = numeric(nRows)
+                        }#,
+                        #FUN.VALUE = numeric(1)
+                    )
+                    pwDistL
+                }#,
+                #FUN.VALUE = numeric(nRows)
             )
             
-            # in case nRows is equal to 1, matrix is 'dropped' to vector
-            if (!is.matrix(pwDistMat)) {
-                pwDistMat <- matrix(pwDistMat, nrow = nRows)
-            }
-
-            pwDistMat
+            pwDistLL
+            
         } 
         
         pwDistByBlock <- BiocParallel::bplapply(
@@ -748,6 +762,11 @@ EMDDist <- function(
             verbose = verbose)
         
     } else {
+        # currently , this portion of code is not used: 
+        # here, the global distance is generated 
+        # in a general fashion and is not necessarily
+        # the sum of uni-dimensional distances
+        # therefore only one distance matrix (portion) is calculated
         handleOneBlock <- function(
         block,
         loadExprMatrixFUN,
@@ -789,6 +808,11 @@ EMDDist <- function(
                         }
                         exprMat <- exprMat[, channels, drop=FALSE]
                     }
+                    # set used feature names if not done yet 
+                    # (for DistSum object output)
+                    if (is.null(usedFeatureNames)) {
+                        usedFeatureNames <- colnames(exprMat)    
+                    }
                     exprMat
                 },
                 loadExprMatrixFUN = loadExprMatrixFUN,
@@ -797,12 +821,13 @@ EMDDist <- function(
                 verbose = verbose
             )
             
-            pwDistMat <- vapply(
-                seq_along(colSeq),
-                FUN = function(j) {
-                    pwDistCol <- vapply(
-                        seq_along(rowSeq),
-                        FUN = function(i) {
+            pwDistLL <- lapply(
+                seq_along(rowSeq),
+                FUN = function(i) {
+                    pwDistL <- lapply(
+                        seq_along(colSeq),
+                        FUN = function(j) {
+                            # only triangular part is calculated
                             if (colSeq[j] > rowSeq[i]) {
                                 pwDist <- .EMDDist(
                                     expr1 = expMatList[[
@@ -823,20 +848,14 @@ EMDDist <- function(
                             } else {
                                 pwDist <- 0.
                             }
+                            names(pwDist) <- "AllChannels"
                             pwDist
-                        },
-                        FUN.VALUE = numeric(1))
-                    pwDistCol
-                },
-                FUN.VALUE = numeric(nRows)
-            )
+                        })
+                    
+                    pwDistL
+                })
             
-            # in case nRows is equal to 1, matrix is 'dropped' to vector
-            if (!is.matrix(pwDistMat)) {
-                pwDistMat <- matrix(pwDistMat, nrow = nRows)
-            }
-            
-            pwDistMat
+            pwDistLL
         }
         
         pwDistByBlock <- BiocParallel::bplapply(
@@ -848,44 +867,56 @@ EMDDist <- function(
             loadExprMatrixFUNArgs = loadExprMatrixFUNArgs,
             channels = channels,
             verbose = verbose)
-        
     }
     
-    # re-arrange block results to create one single matrix
+    # re-arrange block results to create one single matrix (per feature)
     nRows <- rowRange[2] - rowRange[1] + 1
     nCols <- colRange[2] - colRange[1] + 1
-    
-    pwDist <- matrix(rep(0., nRows*nCols), nrow = nRows)
-    
-    for (b in seq_along(blocks2D)){
-        block <- blocks2D[[b]]
-        for (i in seq(block$rowMin, block$rowMax)) {
-            for (j in seq(block$colMin, block$colMax)) {
-                pwDist[i - rowRange[1] + 1, j - colRange[1] + 1] <- 
-                    pwDistByBlock[[b]][i - block$rowMin + 1,
-                                       j - block$colMin + 1]
-            }
-        }
-    }
-    
     rowSeq <- seq(rowRange[1], rowRange[2])
     colSeq <- seq(colRange[1], colRange[2])
     
-    rownames(pwDist) <- rowSeq
-    colnames(pwDist) <- colSeq
     
-    # apply symmetry for lower triangular part of matrix 
-    # NOTE we do it only if rowSeq is identical to colSeq. 
-    # If not, it means we are working on a not symmetrical block, 
-    # hence we might not have the info at our disposal to fill in the parts 
-    # that belong to the low triangle of the distance matrix.
-
-    if (rowRange[1] == colRange[1] && rowRange[2] == colRange[[2]]) {
-        pwDist <- pwDist + t(pwDist)      
-    }
+    pwDistPerDim <- lapply(
+         seq_along(usedFeatureNames),
+         FUN = function(chIndex) {
+             pwDist <- matrix(rep(0., nRows*nCols), nrow = nRows)
+             
+             for (b in seq_along(blocks2D)){
+                 block <- blocks2D[[b]]
+                 for (i in seq(block$rowMin, block$rowMax)) {
+                     for (j in seq(block$colMin, block$colMax)) {
+                         pwDist[i - rowRange[1] + 1, j - colRange[1] + 1] <- 
+                             pwDistByBlock[[b]][[i - block$rowMin + 1]][[j - block$colMin + 1]][chIndex]
+                     }
+                 }
+             }
+             
+             rownames(pwDist) <- rowSeq
+             colnames(pwDist) <- colSeq
+             
+             # apply symmetry for lower triangular part of matrix 
+             # NOTE we do it only if rowSeq is identical to colSeq. 
+             # If not, it means we are working on a not symmetrical block, 
+             # hence we might not have the info at our disposal to fill in the parts 
+             # that belong to the low triangle of the distance matrix.
+             
+             if (rowRange[1] == colRange[1] && rowRange[2] == colRange[[2]]) {
+                 pwDist <- pwDist + t(pwDist)      
+             }
+             
+             pwDist
+         }
+    )
     
-    return(pwDist)
+    # set feature names for use into DistSum object
+    names(pwDistPerDim) <- usedFeatureNames
+    
+    distObj <- DistSum(object = pwDistPerDim)
+    
+    distObj
 }
+    
+    
 
 #' @title Pairwise Earth Mover's Distance calculation
 #' @description Computation of all EMD between pairs of flowFrames belonging
@@ -1477,7 +1508,15 @@ channelSummaryStats <- function(
 #' looking for too big projection spaces. 
 #' @param pwDist (`nSamples` rows, `nSamples` columns), 
 #' previously calculated pairwise distances between samples, 
-#' must be provided as a full symmetric square matrix, with 0. diagonal
+#' can be provided as :
+#' - a `DistSum` object
+#' - a `dist` object
+#' - a full symmetric square matrix, with 0. diagonal
+#' @param whichChannels if `pwDist` has been provided as a `DistSum` object,
+#' a vector of channels to be included in the distances. 
+#' In that case the distances have been computed as a sum of unidimensional 
+#' distances for each channel, and the `DistSum` object allows to restrict 
+#' the channel sets to be included in the distance accounting
 #' @param nDim number of dimensions of projection, as input to SMACOF algorithm
 #' if not provided, will be found iteratively using `targetPseudoRSq`
 #' @param seed seed to be set when launching SMACOF algorithm 
@@ -1487,25 +1526,7 @@ channelSummaryStats <- function(
 #' @param maxDim in case `nDim` is found iteratively, 
 #' maximum number of dimensions the search procedure is allowed to explore
 #' @param ... additional parameters passed to SMACOF algorithm
-#'
-#' @return a list with six elements:
-#' - `$pwDist` the initial pair-wise distance (same as input)
-#' - `$proj` the final configuration, i.e. the projected data matrix 
-#' (`nSamples` rows, `nDim` columns) in `nDim` dimensions
-#' - `$projDist` the distance matrix of projected data
-#' - `stress` the global stress loss function final value 
-#' obtained from the SMACOF algorithm
-#' - `spp` the stress per point obtained from the SMACOF algorithm, i.e.
-#' the contribution of each point to the stress loss function
-#' - `$RSq` R squares, for each d, from 1 to `nDim`:
-#' the (pseudo) R square when taking all dims from 1 to d.
-#' - `$GoF` Goodness of fit, for each d, from 1 to `nDim`:
-#' the goodness of fit indicator (b/w 0 and 1) when taking all dims from 1 to d.
-#' Note pseudo R square and goodness of fit indicators are essentially the 
-#' same indicator, only the definition of total sum of squares differ:
-#' - for pseudo RSq: TSS is calculated using the mean pairwise distance 
-#' as minimum
-#' - for goodness of fit: TSS is calculated using 0 as minimum
+#' @return an object of S4 class `MDS` 
 #' 
 #' @importFrom stats as.dist dist
 #' @export
@@ -1567,25 +1588,33 @@ channelSummaryStats <- function(
 #' 
 computeMetricMDS <- function(
         pwDist,
+        whichChannels = NULL,
         nDim = NULL,
         seed = NULL,
         targetPseudoRSq = 0.95,
         maxDim = 128,
         ...){
-    
-    if (inherits(pwDist, "dist")) {
-        pwDist <- as.matrix(pwDist)
+    if (inherits(pwDist, "DistSum")) {
+        pwDist <- as.matrix(pwDist, whichFeatures = whichChannels)
     } else {
-        dimensions <- dim(pwDist)
-        if (length(dimensions) != 2) {
-            stop("pwDist should be a square matrix")
-        }
-        if(dimensions[1] != dimensions[2]) {
-            stop("pwDist should be a square matrix")
-        }
-        if (!is.numeric(pwDist)) {
-            stop("pwDist should be numeric")
+        if (!is.null(whichChannels)) {
+            warning("whichChannels parameter not taken into account as ",
+                    "`pwDist] is not a `DistSum` object")
         }
+        if (inherits(pwDist, "dist")) {
+            pwDist <- as.matrix(pwDist)
+        } 
+    }
+    
+    dimensions <- dim(pwDist)
+    if (length(dimensions) != 2) {
+        stop("pwDist has more than 2 dimensions, it should be a square matrix")
+    }
+    if(dimensions[1] != dimensions[2]) {
+        stop("pwDist is not square")
+    }
+    if (!is.numeric(pwDist)) {
+        stop("pwDist should be numeric")
     }
     
     # handle case when nDim not provided
