@@ -80,6 +80,18 @@ test_that("ggplotMarginalDensities works", {
         "ggplotMarginalDensities with channels with transList",
         fig = p)
     
+    selChannels <- c("FSC-A", "SSC-A", "BV785 - CD3", "APCCy7 - CD4")
+    p <- ggplotMarginalDensities(
+        OMIP021Samples,
+        channels = selChannels,
+        pDataForGroup = "Donor",
+        pDataForColour = "Donor",
+        transList = transList
+    )
+    vdiffr::expect_doppelganger(
+        "ggplotMarginalDensities with markers with transList",
+        fig = p)
+    
     p <- ggplotMarginalDensities(
         fsAll
     )
@@ -169,6 +181,12 @@ test_that("ggplotMarginalDensities works", {
         fig = p)
 })
 
+test_that("ggplotDistFeatureImportance works", {
+    p <- ggplotDistFeatureImportance(pwDist)
+    vdiffr::expect_doppelganger("ggplotDistFeatureImportance",
+                                fig = p)
+})
+
 test_that("ggplotSampleMDS works", {
 
     mdsObj <- computeMetricMDS(pwDist, nDim = 4, seed = 0)
@@ -433,11 +451,23 @@ test_that("ggplotSampleMDS works", {
                          pDataForColour = "grpId",
                          pDataForLabel = "name",
                          pDataForShape = "type",
-                         sizeReflectingStress = TRUE,
+                         pointSizeReflectingStress = TRUE,
                          seed = 0)
 
     vdiffr::expect_doppelganger("ggplotSampleMDS with sizeReflectingStress",
                                 fig = p)
+    
+    p <- ggplotSampleMDS(mdsObj = mdsObj,
+                         pData = flowCore::pData(fsAll),
+                         projectionAxes = c(1,2),
+                         pDataForColour = "grpId",
+                         pDataForLabel = "name",
+                         pDataForShape = "type",
+                         pointSize = 2,
+                         seed = 0)
+
+    vdiffr::expect_doppelganger("ggplotSampleMDS with pointSize",
+                                fig = p)
 
     # use pData for additional labelling
     p <- ggplotSampleMDS(mdsObj = mdsObj,
@@ -446,8 +476,8 @@ test_that("ggplotSampleMDS works", {
                          pDataForAdditionalLabelling = c("grpId", "type"),
                          repelPointLabels = FALSE)
 
-    expect_equal(p$labels$text, "grpId")
-    expect_equal(p$labels$text2, "type")
+    expect_equal(p$labels$text2, "grpId")
+    expect_equal(p$labels$text3, "type")
 
     # test that pData for additional labelling has been well ignored
     # with biplot activated
