@@ -456,7 +456,7 @@ test_that("pairwiseEMDDist with fs works", {
         minRange = -10,
         maxRange = 10)
     expect_equal(dim(pwDist), c(1,1))
-    expect_equal(pwDist[1,1], 0.)
+    expect_equal(as.matrix(pwDist)[1,1], 0.)
     
     pwDist <- pairwiseEMDDist(
         x = OMIP021Trans,
@@ -464,6 +464,25 @@ test_that("pairwiseEMDDist with fs works", {
         binSize = 0.05,
         minRange = -10,
         maxRange = 10)
+    
+    pwDist <- as.matrix(pwDist)
+    
+    expect_equal(dim(pwDist), c(2,2))
+    expect_equal(pwDist[1,1], 0.)
+    expect_equal(pwDist[1,2], 0.1551)
+    expect_equal(pwDist[2,1], 0.1551)
+    expect_equal(pwDist[2,2], 0.)
+    
+    # now calculate the distance with all channels
+    # and select later
+    pwDist <- pairwiseEMDDist(
+        x = OMIP021Trans,
+        channels = NULL,
+        binSize = 0.05,
+        minRange = -10,
+        maxRange = 10)
+    
+    pwDist <- as.matrix(pwDist, whichFeatures = c("FSC-A", "SSC-A"))
     expect_equal(dim(pwDist), c(2,2))
     expect_equal(pwDist[1,1], 0.)
     expect_equal(pwDist[1,2], 0.1551)
@@ -494,6 +513,9 @@ test_that("pairwiseEMDDist with fs works", {
         minRange = -10,
         maxRange = 10,
         verbose = FALSE)
+    
+    pwDist2 <- as.matrix(pwDist2)
+    
     expect_equal(dim(pwDist2), c(3,2))
     expect_equal(pwDist2[1,1], 0.07241)
     expect_equal(pwDist2[1,2], 0.08347)
@@ -513,6 +535,8 @@ test_that("pairwiseEMDDist with fs works", {
         maxRange = 10,
         verbose = FALSE)
     
+    pwDist3 <- as.matrix(pwDist3)
+    
     expect_equal(dim(pwDist3), c(5,4))
     expect_equal(pwDist3[1,1], 0.1551)
     expect_equal(pwDist3[1,2], 0.07132)
@@ -535,7 +559,9 @@ test_that("pairwiseEMDDist with fs works", {
     expect_equal(length(msg), 22)
     expect_equal(msg[5], "Calculating histogram for file 2...\n")
     expect_equal(msg[10], "Loading file 5...\n")
-    expect_equal(msg[21], "i = 3; j = 5; dist = 0.01813\n")
+    expect_equal(msg[21], "i = 3; j = 5; sum(dist) = 0.01813\n")
+    
+    pwDist4 <- as.matrix(pwDist4)
     
     expect_equal(dim(pwDist4), c(5,4))
     expect_equal(pwDist4[1,1], 0.1551)
@@ -559,15 +585,38 @@ test_that("pairwiseEMDDist with expr matrix works", {
         binSize = 0.05,
         minRange = -10,
         maxRange = 10)
+    
+    pwDist <- as.matrix(pwDist)
+    
     expect_equal(dim(pwDist), c(1,1))
     expect_equal(pwDist[1,1], 0.)
     
+    exprList <- list(sample1 = expr1, sample2 = expr2)
+    
     pwDist <- pairwiseEMDDist(
-        x = list(expr1, expr2),
+        x = exprList,
         channels = c("FSC-A", "SSC-A"),
         binSize = 0.05,
         minRange = -10,
         maxRange = 10)
+    
+    pwDist <- as.matrix(pwDist)
+    
+    expect_equal(dim(pwDist), c(2,2))
+    expect_equal(pwDist[1,1], 0.)
+    expect_equal(pwDist[1,2], 0.1551)
+    expect_equal(pwDist[2,1], 0.1551)
+    expect_equal(pwDist[2,2], 0.)
+    
+    # now calculate the distance with all channels
+    # and select later
+    pwDist <- pairwiseEMDDist(
+        x = list(expr1, expr2),
+        channels = NULL,
+        binSize = 0.05,
+        minRange = -10,
+        maxRange = 10)
+    pwDist <- as.matrix(pwDist, whichFeatures = c("FSC-A", "SSC-A"))
     expect_equal(dim(pwDist), c(2,2))
     expect_equal(pwDist[1,1], 0.)
     expect_equal(pwDist[1,2], 0.1551)
@@ -600,6 +649,9 @@ test_that("pairwiseEMDDist with expr matrix works", {
         minRange = -10,
         maxRange = 10,
         verbose = FALSE)
+    
+    pwDist2 <- as.matrix(pwDist2)
+    
     expect_equal(dim(pwDist2), c(3,2))
     expect_equal(pwDist2[1,1], 0.07241)
     expect_equal(pwDist2[1,2], 0.08347)
@@ -619,6 +671,8 @@ test_that("pairwiseEMDDist with expr matrix works", {
         maxRange = 10,
         verbose = FALSE)
     
+    pwDist3 <- as.matrix(pwDist3)
+    
     expect_equal(dim(pwDist3), c(5,4))
     expect_equal(pwDist3[1,1], 0.1551)
     expect_equal(pwDist3[1,2], 0.07132)
@@ -638,10 +692,12 @@ test_that("pairwiseEMDDist with expr matrix works", {
         maxRange = 10,
         verbose = TRUE))
     
+    pwDist4 <- as.matrix(pwDist4)
+    
     expect_equal(length(msg), 22)
     expect_equal(msg[5], "Calculating histogram for file 2...\n")
     expect_equal(msg[10], "Loading file 5...\n")
-    expect_equal(msg[21], "i = 3; j = 5; dist = 0.01813\n")
+    expect_equal(msg[21], "i = 3; j = 5; sum(dist) = 0.01813\n")
     
     expect_equal(dim(pwDist4), c(5,4))
     expect_equal(pwDist4[1,1], 0.1551)
@@ -669,6 +725,8 @@ test_that("pairwiseEMDDist works with fs and BiocParallel", {
         minRange = -10,
         maxRange = 10,
         BPPARAM = bp))
+    
+    pwDist <- as.matrix(pwDist)
 
     expect_equal(dim(pwDist), c(2,2))
     expect_equal(pwDist[1,1], 0.)
@@ -705,6 +763,8 @@ test_that("pairwiseEMDDist works with fs and BiocParallel", {
             minRange = -10,
             maxRange = 10,
             BPPARAM = bp))
+    
+    pwDist2 <- as.matrix(pwDist2)
 
     expect_equal(dim(pwDist2), c(3,2))
     expect_equal(pwDist2[1,1], 0.07241)
@@ -723,6 +783,8 @@ test_that("pairwiseEMDDist works with fs and BiocParallel", {
         minRange = -10,
         maxRange = 10,
         BPPARAM = bp))
+    
+    pwDist3 <- as.matrix(pwDist3)
 
     expect_equal(dim(pwDist3), c(2,2))
     expect_equal(pwDist3[1,1], 0.07241)
@@ -754,6 +816,8 @@ test_that("pairwiseEMDDist works with expr matrix and BiocParallel", {
         maxRange = 10,
         BPPARAM = bp))
     
+    pwDist <- as.matrix(pwDist)
+    
     expect_equal(dim(pwDist), c(2,2))
     expect_equal(pwDist[1,1], 0.)
     expect_equal(pwDist[1,2], 0.1551)
@@ -792,6 +856,8 @@ test_that("pairwiseEMDDist works with expr matrix and BiocParallel", {
         maxRange = 10,
         BPPARAM = bp))
     
+    pwDist2 <- as.matrix(pwDist2)
+    
     expect_equal(dim(pwDist2), c(3,2))
     expect_equal(pwDist2[1,1], 0.07241)
     expect_equal(pwDist2[1,2], 0.08347)
@@ -810,6 +876,8 @@ test_that("pairwiseEMDDist works with expr matrix and BiocParallel", {
         maxRange = 10,
         BPPARAM = bp))
     
+    pwDist3 <- as.matrix(pwDist3)
+    
     expect_equal(dim(pwDist3), c(2,2))
     expect_equal(pwDist3[1,1], 0.07241)
     expect_equal(pwDist3[1,2], 0.08347)
@@ -839,6 +907,8 @@ test_that("pairwiseEMDDist dynamic memory loading simulation", {
         channels = c("FSC-A", "SSC-A"),
         verbose = FALSE)
     
+    pwDist <- as.matrix(pwDist)
+    
     expect_equal(dim(pwDist), c(10,10))
     expect_equal(pwDist[1,2], 0.07070)
     expect_equal(pwDist[1,7], 0.14340)
@@ -869,6 +939,8 @@ test_that("pairwiseEMDDist dynamic memory loading simulation", {
         BPOPTIONS = BiocParallel::bpoptions(
             packages = c("flowCore", "CytoPipeline"))))
     
+    pwDist2 <- as.matrix(pwDist2)
+    
     # Note it is normal that the computed distances are different 
     # from the ones obtained when not using BiocParallel.
     # As described in section 2.4 of technical note:
@@ -1243,6 +1315,18 @@ test_that("computeMetricMDS works", {
     names(tgtspp) <- 1:5
     expect_equal(spp(mdsObj), tgtspp)
     
+    # do the same but with the full distance object, 
+    # and select channels in computeMetricMDS
+    pwDistFull <- pairwiseEMDDist(fsAll, 
+                                  channels = NULL,
+                                  verbose = TRUE)
+    
+    mdsObjFull <- computeMetricMDS(pwDistFull, nDim = 2, seed = 0,
+                                   whichChannels = c("FSC-A", "SSC-A"))
+    expect_equal(stress(mdsObjFull), 0.0203635387)
+    expect_equal(spp(mdsObjFull), tgtspp)
+    
+    
     # with no user provided nDim, but (implicit) target pseudo rsquare = 0.95
     mdsObj2 <- computeMetricMDS(pwDist, seed = 0)
     
